/***********************************************************************/
/*                                                                     */
/*                         C_data_user_complex.c                       */
/*                                                                     */
/*  example of definition of the C type complex                        */
/*                                                                     */
/*  authors: D. Gaffe and A. Ressouche                                 */
/*                                                                     */
/*  version: 02/01/2019                                                */
/***********************************************************************/

#include <math.h>
#include "C_data_user_complex.h"
#define C_DATA_USER_COMPLEX_EPSILON 0.000000001
const complex zero_complex={0.0,0.0};

complex copy_complex(complex x)
{
  return x;
}

complex add_complex(complex x,complex y)
{
  complex res;
  res.re  = x.re + y.re;
  res.im = x.im + y.im;
  return res;
}

complex sub_complex(complex x,complex y)
{
  complex res;
  res.re  = x.re - y.re;
  res.im = x.im - y.im;
  return res;
}

complex neg_complex(complex x)
{
  complex res;
  res.re  = x.re * (-1.0);
  res.im = x.im;
  return res;
}

complex conj_complex(complex x)
{
  complex res;
  res.re  = x.re;
  res.im = x.im * (-1.0);
  return res;
}
complex mult_complex(complex x,complex y)
{
  complex res;
  res.re  = x.re * y.re - x.im * y.im;
  res.im = x.re * y.im + x.im * y.re;
  return res;
}

complex div_complex(complex x,complex y)
{
  // division by multiplication with conjugated divisor
  complex res, x_tmp, y_tmp, conjy;
  conjy= conj_complex(y);
  x_tmp= mult_complex(x,conjy);
  y_tmp= mult_complex(y,conjy);  /* x/y = x_tmp/y_tmp  but y_tmp is real! */
  res.re= x_tmp.re / y_tmp.re;
  res.im= x_tmp.im / y_tmp.re;
  return res;
}

double module_complex(complex x)
{
  return sqrt(x.re * x.re + x.im * x.im);
}

double argument_complex(complex x)
{
  double res;
  if(! eq_complex(x,zero_complex))
  {
    res= acos(x.re / module_complex(x));
    if((x.im < 0.0) || (x.im > M_PI));
    {
      res= res* -1.0;
    }
  }
  else
  {
    res= 0.0;
  }
  return res;
}

int eq_complex(complex x,complex y)
{
  return (fabs(x.re-y.re) < C_DATA_USER_COMPLEX_EPSILON) &&
         (fabs(x.im-y.im) < C_DATA_USER_COMPLEX_EPSILON);
}

int neq_complex(complex x,complex y)
{
  return (fabs(x.re-y.re) >= C_DATA_USER_COMPLEX_EPSILON) ||
         (fabs(x.im-y.im) >= C_DATA_USER_COMPLEX_EPSILON);
}

int less_complex(complex x,complex y)
{
  return ((x.re < y.re) && (x.im <= y.im))
         ||
         ((x.im < y.im) && (x.re <= y.re));
}

int lesseq_complex(complex x,complex y)
{
  return less_complex(x,y) || eq_complex(x,y);
}

int sup_complex(complex x,complex y)
{
  return ((x.re > y.re) && (x.im >= y.im))
         ||
         ((x.im > y.im) && (x.re >= y.re));
}

int supeq_complex(complex x,complex y)
{
  return sup_complex(x,y) || eq_complex(x,y);
}
